/********************************************************************
                                                               
	COPYRIGHT (C) 2000, Z-World Incorporated, ALL RIGHTS RESERVED 
                                                               
	coldload.c

	This program bootstraps a Rabbit processor system and will load
	itself to ram address 00h at 2400 baud at powerup.  At end of
	program load,	bootstrap is terminated and starts executing
	in RAM.  Memory management registers are configured as indicated
	below	in source code.

	It first detects the crystal frequency by counting the number of
	times a loop executes in a given amount of time measured with
	the RTC.  This determines the time constant for 19200 baud for
	the connected crystal.

	It will change the time constant to ramp up to 19200 baud and
	wait for the pilot BIOS file, where it will place the pilot (max
	816 bytes) to ram address 0x4100.  Pilot contains target
	communication protocol for Dynamic C to load bare bios.
	
	Procedure:
	Rabbit processor takes 3-byte instructions in bootstrap
	operation.  Bit 7 of 1st-byte determines memory or register
	instruction: 0 for memory, 1 for register.
	2nd-byte is memory or register address.
	3rd-byte is data or opcode byte.

	Change revision number at end of this file.
	
	Create coldload binary file by compile type with no bios (*.bpf)
	and compile to file using a Rabbit RTI file.  Strip off excess
	bytes.

	coldload is approximately 450 bytes (530 bytes with version
	text).

*******************************************************************/

//	Defines:
#define  DIVADDR		0x4000	// time constant address
#define  REGBIOSFLAG	0x4001 	// start bare BIOS flag address
#define  FREQADRS		0x4002	// frequency divisor address
#define  PILOTSPOT	0x4100   // pilot BIOS start execution

// locate code at 0000
#rcodorg rootcode	 0x0 0x0 0x6000 apply

#asm
main::
premain::
// configure registersmemory map
	db		80h,000h,008h		;GCSR: no periodic int, main osc no div
	db		80h,010h,000h		; MMIDR: normal operation
	db		80h,014h,005h		; MB0CR: 4 ws, OE1, CS1 active
	db		80h,015h,005h		; MB1CR: 4 ws, OE1, CS1 active
	db		80h,016h,000h		; MB2CR: 4 ws, OE0, WE0, CS0 active
	db		80h,017h,000h		; MB3CR: 4 ws, OE0, WE0, CS0 active
	db		80h,013h,0c6h		; SEGSIZE
	db		80h,011h,074h		; STACKSEG
	db		80h,012h,03ah		; DATASEG

// crystal frequency detection begins here
	db		000h,000h,001h	;ld    bc,0000h        		; our counter
	db		000h,001h,000h
	db		000h,002h,000h

	db		000h,003h,011h	;ld    de,07FFh        		; mask for RTC bits
	db		000h,004h,0FFh
	db		000h,005h,007h
; wait_for_zero:
	db		000h,006h,0D3h	;ioi ld    (RTC0R),a     	; fill RTC registers
	db		000h,007h,032h
	db		000h,008h,002h
	db		000h,009h,000h

	db		000h,00Ah,0D3h	;ioi ld    hl,(RTC0R)		; get lowest two RTC regs
	db		000h,00Bh,02Ah
	db		000h,00Ch,002h
	db		000h,00Dh,000h

	db		000h,00Eh,0DCh	;and   hl,de					; mask off bits

	db		000h,00Fh,020h	;jr    nz,wait_for_zero		; wait until bits 0-9 are zero
	db		000h,010h,0F5h
; timing_loop:
	db		000h,011h,003h	;inc   bc						; increment counter

	db		000h,012h,0C5h	;push  bc						; save counter

	db		000h,013h,006h	;ld    b,20             	; empirical loop value
	db		000h,014h,020h										; (timed for 4 wait states)

	db		000h,015h,021h	;ld    hl,WDTCR
	db		000h,016h,008h
	db		000h,017h,000h
; delay_loop:
	db		000h,018h,0D3h	;ioi ld    (hl),5A			; hit watchdog
	db		000h,019h,036h
	db		000h,01Ah,05Ah

	db		000h,01Bh,010h	;djnz  delay_loop
	db		000h,01Ch,0FBh

	db		000h,01Dh,0C1h	;pop   bc						; restore counter

	db		000h,01Eh,0D3h	;ioi ld    (RTC0R),a			; fill RTC registers
	db		000h,01Fh,032h
	db		000h,020h,002h
	db		000h,021h,000h

	db		000h,022h,0D3h	;ioi ld    hl,(RTC0R)		; get lowest two RTC regs
	db		000h,023h,02Ah
	db		000h,024h,002h
	db		000h,025h,000h

	db		000h,026h,0CBh	;bit   2,h						; test bit 10
	db		000h,027h,054h

	db		000h,028h,028h	;jr    z, timing_loop		; repeat until bit set
	db		000h,029h,0E7h

	db		000h,02Ah,060h	;ld    h,b

	db		000h,02Bh,069h	;ld    l,c

	db		000h,02Ch,011h	;ld    de,0008
	db		000h,02Dh,008h
	db		000h,02Eh,000h

	db		000h,02Fh,019h	;add   hl,de					; add 8 (equiv. to rounding up later)

	db		000h,030h,0FCh	;rr    hl
	db		000h,031h,0FCh	;rr    hl
	db		000h,032h,0FCh	;rr    hl
	db		000h,033h,0FCh	;rr    hl						; divide by 16
	db		000h,034h,07Dh	;ld    a,l    					; this is our divider!
	db		000h,035h,03Dh	;dec   a 

	db		000h,036h,032h	;ld    (DIVADDR),a			; save timer A4 scaling
	db		000h,037h,000h
	db		000h,038h,040h

	db		000h,039h,0D3h	;ioi ld    (TAT4R),a			; set timer A4 running at 19200 baud
	db		000h,03Ah,032h
	db		000h,03Bh,0A9h
	db		000h,03Ch,000h

	db		000h,03Dh,03Ch	;inc   a 

	db		000h,03Eh,032h	;ld    (FREQADRS),a       	; save divisor for later
	db		000h,03Fh,002h
	db		000h,040h,040h

	db		000h,041h,03Eh	;ld    a,01  			
	db		000h,042h,001h

	db		000h,043h,0D3h	;ioi ld    (TACSR),a			; enable timer A with cpuclk/2
	db		000h,044h,032h
	db		000h,045h,0A0h
	db		000h,046h,000h

	db		000h,047h,0AFh	;xor   a 

	db		000h,048h,0D3h	;ioi ld    (SACR),a			; set serial port A async, 8 bit, pport C input
	db		000h,049h,032h
	db		000h,04Ah,0C4h
	db		000h,04Bh,000h

; pollrecbuf0:
	db		000h,04Ch,03Eh	;ld    a,5A 
	db		000h,04Dh,05Ah

	db		000h,04Eh,0D3h	;ioi ld    (WDTCR),a			; hit watchdog
	db		000h,04Fh,032h
	db		000h,050h,008h
	db		000h,051h,000h

	db		000h,052h,0D3h	;ioi ld    a,(SASR)			; check byte receive status
	db		000h,053h,03Ah
	db		000h,054h,0C3h
	db		000h,055h,000h

	db		000h,056h,0CBh	;bit   7,a    
	db		000h,057h,07Fh

	db		000h,058h,028h	;jr    z, pollrecbuf0		; wait until byte received
	db		000h,059h,0F2h

	db		000h,05Ah,0D3h	;ioi ld    a,(SADR)			; get byte
	db		000h,05Bh,03Ah
	db		000h,05Ch,0C0h
	db		000h,05Dh,000h

	db		000h,05Eh,0EEh	;xor   CC
	db		000h,05Fh,0CCh

	db		000h,060h,020h	;jr    nz,pollrecbuf0		; wait until byte = 0xCC
	db		000h,061h,0EAh

	db		000h,062h,0AFh	;xor   a 

	db		000h,063h,032h	;ld    (PILOTSPOT),a			; replace 0xCC with 0x00
	db		000h,064h,000h
	db		000h,065h,041h

	db		000h,066h,011h	;ld    de,0400h				; max size of pilot BIOS
	db		000h,067h,099h
	db		000h,068h,03h

	db		000h,069h,0FDh	;ld    iy,PILOTSPOT+1		; start loading bytes here
	db		000h,06Ah,021h
	db		000h,06Bh,001h
	db		000h,06Ch,041h
; pollrecbuf1:
	db		000h,06Dh,03Eh	;ld    a,5A   
	db		000h,06Eh,05Ah

	db		000h,06Fh,0D3h	;ioi ld    (WDTCR),a			; hit watchdog
	db		000h,070h,032h
	db		000h,071h,008h
	db		000h,072h,000h

	db		000h,073h,0D3h	;ioi ld    a,(SASR)			; check for byte received
	db		000h,074h,03Ah
	db		000h,075h,0C3h
	db		000h,076h,000h

	db		000h,077h,0CBh	;bit   7,a
	db		000h,078h,07Fh

	db		000h,079h,028h	;jr    z,pollrecbuf1			; repeat until byte received
	db		000h,07Ah,0F2h

	db		000h,07Bh,0CCh	;bool  hl						; clear hl
	db		000h,07Ch,06Ch	;ld    l,h    

	db		000h,07Dh,0D3h	;ioi ld    a,(SADR)			; get the next byte
	db		000h,07Eh,03Ah
	db		000h,07Fh,0C0h
	db		000h,080h,000h

	db		000h,081h,0FDh	;ld    (iy+0),a				; save it
	db		000h,082h,077h
	db		000h,083h,000h 

	db		000h,084h,0FDh	;inc   iy 
	db		000h,085h,023h

	db		000h,086h,01Bh	;dec   de						; decrement count

	db		000h,087h,0ECh	;or    hl,de

	db		000h,088h,020h	;jr    nz,pollrecbuf1		; repeat until 0x330 bytes receiver
	db		000h,089h,0E3h

	db		000h,08Ah,0C3h	;jp    PILOTSPOT				; start running at PILOTSPOT
	db		000h,08Bh,000h
	db		000h,08Ch,041h

#endasm
#asm
	db		80h,024h,080h	;SPCR: end bootstrap
#endasm
#asm
	db		00h,00h,00h
	db		"Rabbit Coldloader 1.2,"
	db		"Frequency Detection Version, "
	db		"Z-World, Inc. Copyright 2000"
#endasm
