#define DCVER7X
#memmap root
/*************************************************************
Pilot BIOS source code

This is what Dynamic C uses to load the regular BIOS to RAM

*************************************************************/
// These statements locate the code and data correctly for the
// pilot BIOS
#rcodorg rootcode	 0x0 0x4100 0x1fff apply

#rvarorg rootdata	 0x0 0x5fff 0x800	 apply

#ifdef DCVER7X
root int _PB_Entry();			// entry point for the Pilot BIOS finite state machine
root int _PB_Init() ;			// initilaization for the Pilot BIOS finite state machine
root _PB_WriteMem();
root _PB_ReadProgPort();
root _PB_WriteProgPort();
root _PB_InitTC();
root _PB_SetIntVecTab();
#else
int _PB_Entry();			// entry point for the Pilot BIOS finite state machine
int _PB_Init() ;			// initilaization for the Pilot BIOS finite state machine
#endif

struct _dkcHeader _PBcurHeader; 	//	current header information

#define HIXPC 				0x71
#define LOXPCBASE 		0xF9
#define REGBIOSFLAG		0x4001 	// start bare BIOS flag address
#define FLASH_ID   		0x4006
#define PILOTSPOT 		0x4100
#define DKC_FLASH_HDR	0xA4
#define DKC_RAM_HDR		0xA3
#define DIVADRS 			0x4000
#define BEGREGBIOS 		0x0000
#define _PBRAM_SIZE  	0x4005
#define _PBINTVEC_BASE  0x5000

/******* variable definitions ***********/
char sysStack[0x1ff];
void *_PB_State;				//	this is the address (state) of the comm FSM
char _PBcommBuffer[256];	//	communication buffer

#asm root
premain::
main::
	db		0cch				; loader looks for this byte; then replaces with 00h

	ld		hl,REGBIOSFLAG	; clear start bios flags
	ld		(hl),00h

;**** TEST RAM SIZE ***********
;	Dynamic C needs to know how much RAM is available
	ld 	a,HIXPC			; upper Phys. Addr. byte
	ld    xpc,a
	ld 	hl,0xefff		; save byte at test location
	ld    b,(hl)
	exx
	ld 	hl,0xefff		; initialize logical xmem address

	ld 	c,1				; c counts number of 32k blocks of RAM
	ld 	e,LOXPCBASE		; start at Phys Adr RAMBASE+0x7fff
loop:
	ld 	a,HIXPC			; save upper Phys Adr byte in d
	ld    xpc,a
	ld 	a,(hl)			;
	ld 	d,a				;

	ld    a,e				; read PhysAdr 0x80000+(0x8000*c - 1) 
	ld    xpc,a				;	into a
	ld 	a,(hl)			;

	cp 	d			
	jr    nz, not_it		; can't be this size if not equal,
								;   but need to confirm with another 	
								;   value if equal

	ld 	a,HIXPC			; Invert the bits in the high PA byte  
	ld    xpc,a				;   and see if it changes @
	ld    a,d				;   PhysAdr RAMBASE+(0x8000*c - 1)
	xor   0xff				;
	ld 	(hl),a			;   
	ld 	d,a      	   ;
	ld    a,e				; 
	ld    xpc,a
	ld 	a,(hl)			;
	cp    d					;

	jr    z,done  
	
not_it:
	inc   c
	jr    z,errdone		; if c rolls over something is very wrong
	ld    a,e
	add   a,8				; e has xpc for next 32k block
	ld    e,a
	jr    loop	
	
errdone: 
	ld    c,0
done:
	ld    h,0				; restore  upper PhysAddr byte
	ld    a,c
	ex    af,af'			; save count
	exx
	ld 	a,HIXPC
	ld    xpc,a
	ld 	hl,0xefff
	ld    (hl),b
	ex    af,af'			; a now has count

	ld		hl,_PBRAM_SIZE	; clear start bios flags
	ld		(hl),a
;**** END TEST RAM SIZE ********

	ld    a,0x5A
	ioi   ld (WDTCR),a
	
;**** Get Flash ID *************
; Dynamic C can determine the Flash size and other
; things from the Flash ID. This only works on Atmel
; compatible flash types. 
	ld 	a,0x77
	ld    xpc,a
	ld 	a,0xaa
	ld    (0xe555),a	
	ld 	a,0x74
	ld    xpc,a
	ld 	a,0x55
	ld    (0xeaaa),a	
	ld 	a,0x77
	ld    xpc,a
	ld 	a,0x90
	ld    (0xe555),a	
	ld 	a,0x72
	ld    xpc,a
	ld    a,(0xe000)
	ld    (FLASH_ID+1),a
	ld    a,(0xe001)
	ld    (FLASH_ID),a
	ld 	a,0x77
	ld    xpc,a
	ld 	a,0xaa
	ld    (0xe555),a	
	ld 	a,0x74
	ld    xpc,a
	ld 	a,0x55
	ld    (0xeaaa),a	
	ld 	a,0x77
	ld    xpc,a
	ld 	a,0xf0
	ld    (0xe555),a	
;**** EndGet Flash ID

	ld    a,0x5A
	ioi   ld (WDTCR),a

	ipset 1
	
_PB_SetProgPort:
	;	comm port A
	;	parallel port c, bit 6,7 use alternate pin assignment
	ld 	a,11000000b
	ioi   ld (PCFR),a

	ld		a,0xc3			;	jump instruction
	ld		(_PBINTVEC_BASE+SERA_OFS),a
	ld		hl,_PB_SerialISR 
	ld		(_PBINTVEC_BASE+SERA_OFS+1),hl
		
	;	at port c, async, 8-bit, interrupt priority 1

	ld 	a,00000001b
	ioi   ld (SACR),a

_PB_SetSP:
	;	initialize stack pointer
	ld		hl,sysStack+@LENGTH-1
	ld		sp,hl

_PB_SetComm:
	ld		hl,_PB_IState
	ld		(_PB_State),hl				;	move to initial state
	call	_PB_InitTC
	call	_PB_SetIntVecTab
	ipset 0
	
_PB_Loop:
	ld 	a,0x5a						; hit watchdog
	ioi   ld (WDTCR),a
	jp		_PB_Loop

_PB_InitTC:
	call	_PB_Init						;	initialize comm module
	ret
		
	;	repeat trying until we read a byte from the receive buffer
_PB_ReadAgain:
	ld 	a,0x5a						; hit watchdog
	ioi   ld (WDTCR),a

	call	_PB_ReadProgPort
	jr		z,_PB_ReadAgain

	;	repeat trying until we unload the byte to the transmit buffer
	ld		c,a

_PB_WriteAgain:
	ld 	a,0x5a						; hit watchdog
	ioi   ld (WDTCR),a

	call	_PB_WriteProgPort
	jr		nz,_PB_WriteAgain

_PB_WriteMem::
	ld		a,(_PBcurHeader+address+1)
	cp		0xe0
	jr		c,_PB_WriteMem1			;	less than e0, no need to change NBR

#ifdef DCVER7X
	ld		a,(_PBcurHeader+XPCval)
#else
	ld		a,(_PBcurHeader+CBR)
#endif
	ld    xpc,a

_PB_WriteMem1:
	ld		hl,(_PBcurHeader+address)	;	destination
	ex		de,hl
	ld		hl,_PBcommBuffer			;	source
	ld		a,(_PBcurHeader+length)
	ld		b,0
	ld		c,a

_PBmoveByte:
	ld		a,(hl)
	ld		(de),a
	inc	hl
	inc	de
	dec	c
	jr		nz,_PBmoveByte
	ret

;*********** Serial interrupt handler ************
_PB_SerialISR:
	push	af
	ld    a,xpc
	push	af
	push	bc
	push	de
	push	hl
		
	call	_PB_Entry	 			; enter the FSM
	ioi  	ld a,(SASR)				; check the status
	bit	SS_TPRG_BIT,a			; transmit in progress bit
	jr		nz,_PBreadyToExit		; transmit in progress, don't disable it
	;	transmit is not in progress, clear the transmit buffer empty
	;	flag now
	ioi   ld (SASR),a

_PBreadyToExit:
	pop	hl
	pop	de
	pop	bc
	pop	af
	ld    xpc,a
	pop	af
	ipres
	ret

_PB_ReadProgPort::
	;	destroys A
	;	returns byte read (if any) in A
	;	returns with Z set if nothing is read
		
	;	check if there is anything available
	ioi   ld a,(SASR)
	bit	SS_RRDY_BIT,a			; if a received byte ready?
	ret	z							; nope, return with z set
	;	otherwise, a byte *is* ready, read from data port
	ioi   ld a,(SADR)
	ret								; return with z *not* set

_PB_WriteProgPort::
	;	assumes byte to transmit is in C
	;	destroys A
	;	returns with Z reset if not transmitted
		
	;	check if the port is ready
	ioi   ld a,(SASR)

	bit	SS_TFULL_BIT,a			; can I transmit now?
	ret	nz							; nope, return with nz set
	; otherwise, the transmit buffer is ready, write to it!
	ld		a,c						; move byte to transmit to a
	ioi   ld (SADR),a
	ret								; return with z *not* set

_PB_SetIntVecTab::
	;	set up interrupt vector table
	ld		a,0xff & (_PBINTVEC_BASE >> 8)	;	R register has 0x20, so interrupt table starts at 2000
	ld		iir,a
	ld		eir,a
	ret

_PB_Init::	;	initialize the communication module
	ld		hl,_PB_IState
	ld		(_PB_State),hl
	ret

_PB_Read:		; this is the abstraction of the actual read mechanism
	jp		_PB_ReadProgPort

_PB_Write:  	; this is the abstraction of the actual write mechanism
	jp		_PB_WriteProgPort


//****** Begin Pilot BIOS FSM ****************************

_PB_Entry: 	;this is the entry point of the communication FSM
	ld		hl,(_PB_State)						; load address of current state
	jp		(hl)					  				; and jump indirect to it

_PB_IState:	;	this is the initial state
	call	_PB_Read								; if there anything available to read?
	ret	z										; nope, return

	;	we've got something read into register A now, see what it is
	ld		(_PBcurHeader+chksum),a			; initialize checksum
	cp		DKC_READ_HDR						; compare with read header
	jr		nz,_PB_IState1
	ld		hl,_PB_RHdrCnt
	ld		(_PB_State),hl						; advance state to read header count byte
	ret
		
_PB_IState1:
	cp		DKC_WRITE_HDR
	jr		nz,_PB_IState2
	ld		hl,_PB_WHdrCnt
	ld		(_PB_State),hl						; advance state to write header count byte
	ret
	
_PB_IState2:
	cp		DKC_POLL_HDR
	jr		nz,_PB_IState3
	ld		hl,_PB_PHdr
	ld		(_PB_State),hl						; advance state to return flags
	jp		_PB_PHdr
		
_PB_IState3:
	cp		DKC_RAM_HDR
	jr		nz,_PB_IState4
	ld		hl,_PB_RAMHdr
	ld		(_PB_State),hl						; advance state to return flags
	jp		_PB_RAMHdr

_PB_IState4:
	cp		DKC_FLASH_HDR
	jr		nz,_PB_IState5
	ld		hl,_PB_FlashHdr
	ld		(_PB_State),hl						; advance state to return flags
	jp		_PB_FlashHdr

_PB_IState5:
	ret											; just ignore unknown header bytes

_PB_SvStAddChk:					 			; a common point to save state (from hl) and add
													; to checksum (from A)
	ld		(_PB_State),hl						; advance state to read LSB of address
	ld		hl,_PBcurHeader+chksum			; address of checksum
	add	a,(hl)								; add new byt to checksum
	ld		(hl),a								; store back to checksum
	ret
				
_PB_RHdrCnt:	;	now receive the (count) header of a read packet (from target to PC)
	call	_PB_Read
	ret	z										; no byte available, return

	ld		(_PBcurHeader+count),a			; byte read is count, put into structure
	ld		hl,_PB_RHdrA0
	jp		_PB_SvStAddChk
		
_PB_RHdrA0:	;	now receive the (LSB of addr) header of a read packet
	call	_PB_Read
	ret	z										; no byte available, return

	ld		(_PBcurHeader+address),a		; byte read is count, put into structure
	ld		hl,_PB_RHdrA1
	jp		_PB_SvStAddChk
		
_PB_RHdrA1:	;	now receive the (MSB of addr) header of a read packet
	call	_PB_Read
	ret	z										; no byte available, return

	ld		(_PBcurHeader+address+1),a		; byte read is count, put into structure
	ld		hl,_PB_RHdrCBR
	jp		_PB_SvStAddChk
		
_PB_RHdrCBR:	;	now receive the (XPCval) header of a read packet
	call	_PB_Read
	ret	z										; no byte available, return

#ifdef DCVER7X
	ld		(_PBcurHeader+XPCval),a
#else
	ld		(_PBcurHeader+CBR),a
#endif

//	ld		(_PBcurHeader+XPCval),a				; byte read is count, put into structure
	ld		hl,_PB_RHdrChk
	jp		_PB_SvStAddChk
		
_PB_RHdrChk:	;	now receive the (checksum) header of a read packet
	call	_PB_Read
	ret	z										; no byte available, return

	ld		hl,_PBcurHeader+chksum
	cp		(hl)									; compare with checksum
	ld		a,0									; do not replace with xor! need to preserve flags
	ld		(hl),a								; clear checksum
	jr		z,_PB_RHdrChk1
	ld		hl,_PB_RRplyNAK					; doesn't match! sendNAK
	ld		(_PB_State),hl
	jp		_PB_RRplyNAK						; this line obsoletes the following two lines
													; because there is no interrupts to drive the
													; state machine to get to the next state
_PB_RHdrChk1:
	;	matches, now proceed to reply
	ld		hl,_PB_RRplyACK
	ld		(_PB_State),hl
	jp		_PB_RRplyACK						; this line obsoletes the following two lines
													; because there is no interrupts to drive the
													; state machine to get to the next state
		
_PB_RRplyACK::		;	now reply an ACK
	ld		c,DKC_ACK_FLAG
	call	_PB_Write
	ret	nz		;	nothing is written, just return
	ld		hl,_PB_RRplyBytes
	ld		(_PB_State),hl
	ret
		
_PB_RRplyBytes:	;	now reply the contents
	;	now set up the address and optionally XPCval
	ld		a,(_PBcurHeader+address+1)
	cp		0xe0
	jr		c,_PB_RRplyB1						; no need to change NBR
#ifdef DCVER7X
	ld		a,(_PBcurHeader+XPCval)
#else
	ld		a,(_PBcurHeader+CBR)
#endif

//	ld		a,(_PBcurHeader+XPCval)
	ld    xpc,a

_PB_RRplyB1:
	ld		hl,(_PBcurHeader+address)
	ld		c,(hl)
	call	_PB_Write
	ret	nz										; write failed, return

	;	write succeeded
	inc	hl										;	
	ld		(_PBcurHeader+address),hl		; increment address
	ld		hl,_PBcurHeader+chksum			; hl now contains address to checksum
	ld		a,c									; load byte transmitted to A
	add	a,(hl)								; add A to checksum
	ld		(hl),a								; and store back to checksum
	ld		hl,_PBcurHeader+count
	dec	(hl)									; decrement count
	jr		z,_PB_RRplyB2
	ret											; not end of packet yet

_PB_RRplyB2:
	;	end of packet!
	ld		hl,_PB_RRplyChk
	ld		(_PB_State),hl						; advance to next state to output the checksum
	ret
		
_PB_RRplyChk:									; now reply the checksum
	ld		a,(_PBcurHeader+chksum)
	ld		c,a
	call	_PB_Write
	ret	nz										; write failed

	;	the read request is now successfully handled, go back to initial state
	ld		hl,_PB_IState
	ld		(_PB_State),hl
	ret
		
_PB_RRplyNAK:									; reply an NAK
	ld		c,DKC_NAK_FLAG
	call	_PB_Write
	ret	nz
	ld		hl,_PB_IState
	ld		(_PB_State),hl
	ret

_PB_WHdrCnt:									; now receive the (count) header of a write packet (from PC to target)
	call	_PB_Read
	ret	z										; no byte available, return
	ld		(_PBcurHeader+count),a			; byte read is count, put into structure
	ld		(_PBcurHeader+length),a			; also store at length
	ld		hl,_PB_WHdrA0
	jp		_PB_SvStAddChk
		
_PB_WHdrA0:	;	now receive the (LSB of addr) header of a write packet
	call	_PB_Read
	ret	z		 								; no byte available, return

	ld		(_PBcurHeader+address),a		; byte read is count, put into structure
	ld		hl,_PB_WHdrA1
	jp		_PB_SvStAddChk
		
_PB_WHdrA1:	;	now receive the (MSB of addr) header of a write packet
	call	_PB_Read
	ret	z										; no byte available, return

	ld		(_PBcurHeader+address+1),a		; byte read is count, put into structure
	ld		hl,_PB_WHdrCBR
	jp		_PB_SvStAddChk
		
_PB_WHdrCBR:	;	now receive the (XPCval) header of a write packet
	call	_PB_Read
	ret	z										; no byte available, return

#ifdef DCVER7X
	ld		(_PBcurHeader+XPCval),a
#else
	ld		(_PBcurHeader+CBR),a
#endif

//	ld		(_PBcurHeader+XPCval),a				; byte read is count, put into structure
	ld		hl,_PB_WHdrChk
	jp		_PB_SvStAddChk
		
_PB_WHdrChk:		;	now receive the (checksum) header of a write packet
	call	_PB_Read
	ret	z										; no byte available, return

	ld		hl,_PBcurHeader+chksum
	cp		(hl)									; compare with checksum
	jr		z,_PB_WHdrChk1

	;	doesn't match! sendNAK
	ld		hl,_PB_WRplyNAK
	ld		(_PB_State),hl
	jp		_PB_WRplyNAK						; jump directly, there is no interrupt to
											   	; give a tick to the WRplyNAK state
_PB_WHdrChk1:
	;	matches, now proceed to reply
	ld		hl,_PB_WRplyACK
	ld		(_PB_State),hl
	jp		_PB_WRplyACK						; jump directly, there is no interrupt to
													; give a tick to the WRplyACK state
		
_PB_WRplyACK:	;	now reply an ACK
	ld		c,DKC_ACK_FLAG
	call	_PB_Write							; try to write the acknowledge flag
	ret	nz										; return if the UART is still busy
	ld		hl,_PB_WGetBytes					; next, change state to receive bytes
	ld		(_PB_State),hl
	ld		hl,_PBcommBuffer
	ld		(_PBcurHeader+cPointer),hl		; store pointer
	xor	a
	ld		(_PBcurHeader+chksum),a			; clear checksum
	ret
		
_PB_WRplyNAK:	;	reply an NAK
	ld		c,DKC_NAK_FLAG
	call	_PB_Write							; try to write the nak flag
	ret	nz										; return if the UART is still busy
	ld		hl,_PB_IState						; next, return to the initial state
	ld		(_PB_State),hl
	ret
		
_PB_WGetBytes:	; now receive the bytes from the PC
	call	_PB_Read								; try to read a byte
	ret	z										; return if UART doesn't have one ready
	ld		hl,(_PBcurHeader+cPointer)	   ; load pointer to store byte
	ld		(hl),a
	inc	hl
	ld		(_PBcurHeader+cPointer),hl		; increment pointer
	ld		hl,_PBcurHeader+chksum			; hl points to checksum
	add	a,(hl)								; add checksum to A
	ld		(hl),a								; store A as new checksum
	ld		hl,_PBcurHeader+count
	dec	(hl)									; decrement count
	jr		z,_PB_WGetB1						; jump if this is the last byte

	;	nope, not the last byte yet
	ret

_PB_WGetB1:
	;	last byte received, go get the checksum
	ld		hl,_PB_WGetChk
	ld		(_PB_State),hl
	ret
		
_PB_WGetChk:	;	now receive the checksum from the PC
	call	_PB_Read
	ret	z										; try to read the checksum, if not available, return

	;	now we have the sum in A
	ld		hl,_PBcurHeader+chksum
	cp		(hl)									; compare the checksum
	jr		z,_PB_WGetC1

	;	not matched, next state is to transmit an NAK
	ld		hl,_PB_WFinalNAK
	ld		(_PB_State),hl
	jp		_PB_WFinalNAK			 			; jump directly, transition from read to
													; write action has no interrupt to drive
_PB_WGetC1:
	;	matched, next state is to transmit an ACK
	ld		hl,_PB_WFinalACK
	ld		(_PB_State),hl
		
_PB_WFinalACK:	; acknowledge the bytes
	ld		c,DKC_ACK_FLAG
	call	_PB_Write							; attempt to write
	ret	nz										; write is not successful, try again later

	;	write is successful!
	;	checksum matched, copy the bytes now
	call	_PB_WriteMem						; write to memory now!
	ld		hl,_PB_BiosChk						; return to initial state
	ld		(_PB_State),hl
	ret
		
_PB_WFinalNAK:	; nak the bytes
	ld		c,DKC_NAK_FLAG
	call	_PB_Write					  		; attempt to write
	ret	nz										; write failed

	;	write is successful
	ld		hl,_PB_IState						; return to initial state
	ld		(_PB_State),hl
	ret

_PB_BiosChk:
	ld		a,(DIVADRS)		 					; reload time constant for baud changes
	ioi	ld 	(TAT4R),a
	ld		hl,_PB_IState						; return to initial state
	ld		(_PB_State),hl
	ld		hl,REGBIOSFLAG 					; check for regular bios flag
	ld		a,(hl)
	cp		0
	jp		nz,BEGREGBIOS						; flag set so start bios
	ret
	
_PB_PHdr:		; now receive the header of a poll packet (from target to PC)
_PB_RplyChk:	; and the checksum for the flags
	ld		a,(_PBcurHeader+chksum)
	cpl
	ld		c,a
	call	_PB_Write
	ret	nz
	ld		hl,_PB_IState
	ld		(_PB_State),hl
	ret

_PB_RAMHdr:		
	ld		hl,_PB_RplyRAMSize				; next state
	ld		(_PB_State),hl
_PB_RplyRAMSize:								; now reply with the flags
	ld    a,(_PBRAM_SIZE)
	ld    c,a
	ld		(_PBcurHeader+chksum),a
	call	_PB_Write
	ret	nz
	ld		hl,_PB_RplyChk
	ld		(_PB_State),hl
	ret

_PB_FlashHdr:		
	ld		hl,_PB_RplyFlashMID				; next state
	ld		(_PB_State),hl

_PB_RplyFlashMID:			
	ld    a,(FLASH_ID)
	ld    c,a
	ld		(_PBcurHeader+chksum),a
	call	_PB_Write
	ret	nz
	ld		hl,_PB_RplyFlashPID				; next state
	ld		(_PB_State),hl

_PB_RplyFlashPID:			
	ld    a,(FLASH_ID+1)
	ld    c,a
	ld    hl,_PBcurHeader+chksum
	add   a,(hl)
	ld    (_PBcurHeader+chksum),a
	call	_PB_Write
	ret	nz

	ld		hl,_PB_RplyChk 
	ld		(_PB_State),hl
	ret
#endasm
